/*
  Copyright Red Hat, Inc. 2008

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; version 2.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to the
  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
  MA 02139, USA.
*/
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <limits.h>
#include <sys/mount.h>
#include "metadata.h"
#include <string.h>
#include <mntent.h>
#include <unistd.h>
#include <sys/reboot.h>
#ifdef GETTEXT
#include <gettext.h>
#define _(args) gettext(args)
#else
#define _(args) args
#endif

#include "fsc.h"
#include "res-ocf.h"
#include "device.h"

int
call_mount(char *real_device, char *mountpoint, char *fstype, char *options)
{
	char command[PATH_MAX];
	int x;
	 
	if (!options || !strlen(options)) {
		x = snprintf(command, sizeof(command),
			     "mount %s -t %s %s",
			     real_device, fstype, mountpoint);
	} else {
		x = snprintf(command, sizeof(command),
			     "mount %s -t %s -o %s %s",
			     real_device, fstype, options, mountpoint);
	}

	if (x >= (sizeof(command)-1)) {
		printf(_("Command line too long\n"));
		return 1;
	}

	printf(_("FSC [ Calling '%s' ]\n"), command);

	return system(command);
}


int
call_umount(char *mountpoint)
{
	char command[PATH_MAX];
	int x;

	x = snprintf(command, sizeof(command),
		     "umount %s", mountpoint);
	if (x >= (sizeof(command)-1)) {
		printf(_("Command line too long\n"));
		return 1;
	}

	printf(_("FSC [ Calling '%s' ]\n"), command);

	return system(command);
}


int 
main(int argc, char **argv)
{
	char fsname[64];
	char fstype[64];
	char real_device[PATH_MAX];
	char mountpoint[PATH_MAX];
	char options[PATH_MAX];
	char fsopts[PATH_MAX];
	char tmp_buf[64];
	void *fs_flags = NULL;
	char device[PATH_MAX];
	int x, external_mount = 0, self_fence = 0;
	unsigned long mount_flags;
	mount_entries_t *ml = NULL;
	char *lang = NULL;

	snprintf(options, sizeof(options), "PATH=/bin:/sbin");
	if (putenv(options) != 0) {
		printf(_("FSC [ putenv(%s) failed: %s ]\n"),
		       options, strerror(errno));
		return 1;
	}
	memset(options, 0, sizeof(options));

	lang = getenv("LC_ALL");
	if (lang == NULL)
		lang = getenv("LC_MESSAGES");
	if (lang == NULL)
		lang = getenv("LANG");
	if (lang == NULL)
		lang = "C";
	
	if (argc < 2) {
		printf(_("usage: %s { "), argv[0]);
		for (x = 0; fsc_agent.ra_acts[x].a_name; x++) {
			if (fsc_agent.ra_acts[x].a_depth)
				continue;
			printf("%s ", fsc_agent.ra_acts[x].a_name);
		}
		printf("}\n");
		return 1;
	}

	if (!strcasecmp(argv[1], "meta-data")) {
		xml_print_metadata(&fsc_agent);
		return 0;
	}

	x = get_param(&fsc_agent, "name", fsname, sizeof(fsname));
	if (x != 0) {
		printf(_("File system name is not specified.\n"));
		return OCF_RA_INVALID_ARG;
	}

	x = get_param(&fsc_agent, "device", device, sizeof(device));
	if (x != 0) {
		printf(_("File system device or label is not specified\n"));
		return OCF_RA_INVALID_ARG;
	}

	x = get_param(&fsc_agent, "mountpoint", mountpoint, sizeof(mountpoint));
	if (x != 0) {
		printf(_("Mount point is not specified\n"));
		return OCF_RA_INVALID_ARG;
	}

	x = get_param(&fsc_agent, "external_mount", tmp_buf,
		      sizeof(tmp_buf));
	if (x == 0) {
		external_mount = atoi(tmp_buf);
		if (external_mount < 0) {
			printf(_("Invalid value for external_mount\n"));
			return OCF_RA_INVALID_ARG;
		}
	}

	x = get_param(&fsc_agent, "self_fence", tmp_buf,
		      sizeof(tmp_buf));
	if (x == 0) {
		self_fence = atoi(tmp_buf);
		if (external_mount < 0) {
			printf(_("Invalid value for self_fence\n"));
			return OCF_RA_INVALID_ARG;
		}
	}

	x = get_param(&fsc_agent, "fstype", fstype, sizeof(fstype));
	if (x != 0) {
		printf(_("File system type is not specified\n"));
		return OCF_RA_INVALID_ARG;
	}

	x = get_param(&fsc_agent, "options", options, sizeof(options));
	if (x == 0) {
		/* Parse options */
		if (parse_mount_flags(options, &mount_flags,
				      fsopts, sizeof(fsopts)) < 0) {
			printf(_("Mount options are incorrect\n"));
			return OCF_RA_INVALID_ARG;
		}
		fs_flags = fsopts;
	}

	if (!strcasecmp(argv[1], "start")) {
		if (find_fs(device, real_device, sizeof(real_device)) != 0) {
			printf(_("FSC [Could not match %s to a real "
			       "device during start = FAIL ]\n"), device);
			perror("find_fs");
			return OCF_RA_INVALID_ARG;
		}

		ml = get_mount_list("/proc/mounts");
		if (!ml) {
			perror("get_mount_list");
			return OCF_RA_ERROR;
		}

		if (is_mounted(ml, real_device, mountpoint) > 0) {
			printf(_("FSC [%s is already mounted during "
			       "start = OK ]\n"), real_device);
			/* File system already mounted! */
			return OCF_RA_SUCCESS;
		}


		if (external_mount) {
			x = call_mount(real_device, mountpoint, fstype,
				       options);
		} else {
			printf(_("FSC [ Notice: mount(SYSCALL) %s %s %s "
			         "mflags=%08lx fsflags=%s]\n"),
			       real_device, mountpoint, fstype, mount_flags,
			       (char *)fs_flags);
			x = mount(real_device, mountpoint, fstype,
				  mount_flags, (const void *)fs_flags);
		}
		if (x == 0) {
			return OCF_RA_SUCCESS;
		}

		perror("mount");
		return OCF_RA_ERROR;

	} else if (!strcasecmp(argv[1], "stop")) {
		if (find_fs(device, real_device, sizeof(real_device)) != 0) {
			printf(_("FSC [ Could not match %s to a real device"
			         " during stop = OK ]\n"), device);
			return OCF_RA_SUCCESS;
		}

		ml = get_mount_list("/proc/mounts");
		if (!ml) {
			perror("get_mount_list");
			printf(_("FSC [ /proc not mounted?! ]\n"));
			return OCF_RA_ERROR;
		}

		if (is_mounted(ml, real_device, mountpoint) == 0) {
			printf(_("FSC [ %s not mounted during stop = OK ]\n"),
			       real_device);
			/* File system not mounted! */
			return OCF_RA_SUCCESS;
		}

		if (external_mount) {
			x = call_umount(mountpoint);
		} else {
			printf(_("FSC [ Notice: umount %s ]\n"), mountpoint);
			x = umount(mountpoint);
		}
		if (x == 0) {
			return OCF_RA_SUCCESS;
		}

		perror("umount");
		if (self_fence) {
			printf(_("FSC [ Critical: Unmount failed; "
			       "reboot in 5 seconds ]\n"));
			sleep(5);
			reboot(RB_AUTOBOOT);
		}
		return OCF_RA_ERROR;
	} else if (!strcasecmp(argv[1], "status")) {
		if (find_fs(device, real_device, sizeof(real_device)) != 0) {
			printf(_("FSC [ Could not match %s to a real "
			         "device during status = FAIL ]\n"), device);
			return OCF_RA_ERROR;
		}

		ml = get_mount_list("/proc/mounts");
		if (!ml) {
			perror("get_mount_list");
			return OCF_RA_ERROR;
		}

		if (is_mounted(ml, real_device, mountpoint) == 0) {
			printf(_("FSC [ %s is not mounted during status "
			       "= FAIL ]\n"), real_device);
			/* File system not mounted! */
			return OCF_RA_ERROR;
		}
		printf(_("FSC [ %s is mounted = OK ]\n"), real_device);

		return OCF_RA_SUCCESS;
	}

	return 0;
}	
